#!/usr/bin/env python3
#!/nlp/projekty/metatrans2/public_html/python3/bin/python3

import re
import sys

from agenda import Agenda
from chart_parser import ChartParser
from edge import Edge
from log import log


class TopDownChartParser(ChartParser):
    def prepare_agenda(self):
        self.agenda = Agenda(self.known_edges, self.options,
                             (Edge(predicted_by_rule=rule,
                                   how_created='agenda       %s') for rule
                              in self.rules.by_phrase[self.root]))

    def propose_edges(self, agenda_edge):
        # TODO: zrychlení by se dalo dosáhnout na úkor paměti – prostě by si
        # chart ještě pamatoval slovník („index“) levých stran a pod ním
        # seznam – aby se nemusely procházet všechny hrany
        # TODO: a ještě jedna věc: bylo by fajn mít chart víceúrovňový:
        # seznam seznamů hran podle začátku, aby se vybíraly jen ty, které
        # opravdu sedí začátkem na konec uzavřené hrany z agendy
        # TODO: víceúrovňový jen jeden chart (který?), anebo oba?
        # merge, join, append, close, closing, completing, finalizing, extend
        if agenda_edge.closed:
            for chart_edge in self.chart.open_edges:
                if agenda_edge.extends(chart_edge):
                    log.debug('appending to %s', chart_edge)
                    yield chart_edge.extend_with(agenda_edge,
                                                 'merged into  %s')
        else:
            for chart_edge in self.chart.closed_edges:
                if chart_edge.extends(agenda_edge):
                    log.debug('extending w/ %s', chart_edge)
                    yield agenda_edge.extend_with(chart_edge,
                                                  'extended to  %s')

            for extended_edge in self._read_terminal(agenda_edge):
                yield extended_edge

        if (not agenda_edge or agenda_edge.closed or (
                agenda_edge.closed and
                agenda_edge.end == len(self.tokens))):
            # log.debug('             not predicting further from %s',
            #           agenda_edge)
            return

        for predicted_edge in self._predict(agenda_edge):
            yield predicted_edge

    def _read_terminal(self, agenda_edge):
        """
        if E is in the form of [A → α • aj+1 β, i, j]
        create an edge [A → α aj+1 • β, i, j+1].
        """
        if agenda_edge.end >= len(self.tokens):
            return

        if self._match_terminal(agenda_edge, self.tokens[
                agenda_edge.end]):
            yield Edge(extended_from=agenda_edge, tokens=self.tokens,
                       how_created='terminal     %s')

    def _match_terminal(self, agenda_edge, token):
        # log.debug("try match    %s = %s", agenda_edge.current_symbol, token)
        # TODO: tady nemůžu pracovat s "?" (optional), to by fakt chtělo
        # ukládat jinak!
        for attr, expected_value in agenda_edge.current_symbol.items():
            if expected_value is None:
                continue
            elif attr not in token:
                if attr != 'phrase':
                    log.debug("no attr      %s has no %s=%s", token, attr,
                              expected_value)
                return False
            elif token[attr] is None:
                # log.debug("null attr    %s (expected: %s)", attr,
                #           expected_value)
                raise AssertionError('Attribute %s is None in %s, not %s' % (
                    attr, token, expected_value))
            try:  # elif isinstance(token[attr], str):
                if not re.fullmatch(expected_value, token[attr]):
                    # log.debug("mismatch     %s=%s (expected: %s)", attr,
                    #           token[attr], expected_value)
                    return False
            except TypeError:  # else:
                for value in token[attr]:
                    if re.fullmatch(expected_value, value):
                        break
                else:
                    log.debug("mismatch     %s=%s (expected: %s)", attr,
                              token[attr], expected_value)
                    return False

        log.debug("match        %s = %s", token, agenda_edge.current_symbol)
        return True

    # TODO: fakt není potřeba předpovídat všechno, můžu dělat look-up:
    # pokud je první symbol předpovídaného pravidla povinný, musí sedět
    # na příští terminál, jinak by se hrana neměla na co upnout
    # TODO: a samozřejmě ten první symbol nesmí bejt (PHRASE), musí jít
    # o terminální symbol (neterminály radši nebudu řešit, ty třeba ještě
    # nemusej existovat)
    # TODO: samozřejmě ale pořád můžu dělat prořezání pravidel ještě před
    # analýzou, ta taky pomůže, ale oboje prospěje výkonu
    def _predict(self, agenda_edge):
        """
        if E is in the form of [A → α • B β, i, j]
        then for each grammar rule B → γ ∈ P, create an edge [B → • γ, j, j].
        """
        phrase = agenda_edge.left['phrase']
        left_recursive = (agenda_edge.position == 0 and
                          phrase == agenda_edge[0].get('phrase'))
        if left_recursive:
            return

        for rule in self.rules.by_phrase[
                agenda_edge.current_symbol.get('phrase')]:
            # TODO: chce to použít _match_, protože z (NP g=I n=S c=1) nechci
            #                              predikovat (NP c=4) → (k=1 c=4)
            predicted = Edge(predicted_from=agenda_edge,
                             predicted_by_rule=rule,
                             how_created='predict      %s')
            # log.warning('predicted    %s from %s', predicted,
            #             agenda_edge.current_symbol)
            yield predicted

if __name__ == '__main__':
    top_down = TopDownChartParser(argv=sys.argv)
    top_down.parse_from_vertical()
