# věty

from rules import Rules
from .cz_verbals import verbals

# TODO: cz_infinite možná nebude potřeba, možná stačí použít (CLAUSE m=F)
#       a nevkládat podmět(nou větu)

# do slovesné fráze (ale ne před ni!) se vloží nepovinné konstituenty
verbals_with_optionals = Rules(verbals.insert_into_all(
    '(OPTIONALS)', inside=True, after=True))

# indikativní věty mají na začátku (už povinný) konstituent (a za klitiky zase
# nepovinné)
indicative_clauses_initial = Rules(verbals_with_optionals.insert_into_all(
    '(CONSTITUENT) (OPTIONALS)', before=True))
indicative_clauses = Rules(indicative_clauses_initial + verbals_with_optionals)

# všechny věty mohou mít klitika a interpozice, ale pozor, vedlejší už spíš
# ne před podřadícím výrazem (a ani tečku), takže je nebudu míchat
indicative_clauses_initial_clitics = Rules(
    indicative_clauses_initial.insert_into_all(
        '(CLITICS)', position=1))
indicative_clauses_with_clitics = Rules(indicative_clauses.insert_into_all(
    '(CLITICS)', position=1))

indicative_clauses_with_interpositions = Rules(
    indicative_clauses_with_clitics.insert_into_all(
        '(INTER)', before=True, inside=True, after=True))

terminated_indicative_clauses = Rules((
    rule + '(OPT_FULLSTOP)'
    for rule in indicative_clauses_with_interpositions))

# přívlastkové vedlejší věty
relatives = [
    '(RELATIVE g n c) → (lemma=který|jenž|co|jaký g n c)',
    '(RELATIVE n c) → (lemma=co n c)',
    '(RELATIVE g n c) → (k=7 c) (lemma=který|jenž|jaký g n c)',
]

# TODO: jenže mám i kostru věty, kde se shoduje přísudek s hlavou – takže
# „kterého se bála“ by neprošlo, chce to i bez shody – odstranit z hlavy (g n),
# proto bude lepší replace_left!
# TODO: a naopak „který se bál“ (vidíš ten pád?) bude chtít shodu, takže
# vlastně to chce (RELATIVE), které se shoduje s hlavní větou, a druhé, které
# se shoduje v rámci své věty – jde to, doufám? ;-)
attributive_clauses = Rules(verbals_with_optionals.insert_into_all(
    '(RELATIVE g n) (OPTIONALS)', before=True,
    update_left='(ATTR_CLAUSE g n)'))
attributive_clauses = Rules(attributive_clauses.remove_attributes(
    '(g n)', except_position=0, repeat_self=True))
attributive_clauses = Rules(attributive_clauses.insert_into_all(
    '(CLITICS)', position=1))
# „kterého bodla včela“ – bez shody s přísudkem
# „který se smál“ – se shodou, ale zatím to asi není potřeba
#                  (interpretace se z toho dostane i tak)
# souřadné spojení přívlastkových vět je tak trochu nerozlišitelné od hlavní
# věty, když se neopakuje relativum, teda pokud nám někdo nepomůže čárkama :-/
# … a který si na to pak ani nevzpomínal,
# … a dokonce si na to ani nevzpomínal,

# že přijde pozdě, to nám opomněl předem sdělit
# že přijde pozdě, nám opomněl předem sdělit
object_clauses_preinitial = Rules(
    indicative_clauses_initial_clitics.insert_into_all(
        '(word=že)', before=True, update_left='(OBJ_CLAUSE)'))

object_clauses_initial = Rules(verbals_with_optionals.insert_into_all(
    '(word=že) (OPTIONALS)', before=True, update_left='(OBJ_CLAUSE)'))
object_clauses_initial = Rules(object_clauses_initial.insert_into_all(
    '(CLITICS)', position=1))

subordinate_clauses = Rules(attributive_clauses + object_clauses_preinitial +
                       object_clauses_initial)
subordinate_clauses = Rules(subordinate_clauses.insert_into_all(
    '(INTER)', inside=True))
subordinate_clauses = Rules(subordinate_clauses.insert_into_all(
    '(OPT_COMMA)', before=True, after=True))
# TODO: předmětové vedlejší věty
#
# vypadá to, že to není úplně jednoduchý
#   *že utekli jsme z hodiny
#   že jsme utekli z hodiny (iniciální, postiniciální)
#   že ten kluk se ulil z hodiny (preiniciální, iniciální, postiniciální)
#   že se ulil z hodiny (iniciální, postiniciální)
#

# že/co jsem (to) nevěděl
# že maminka se (asi) ztratila

# Kdo	kdo	k3yRnSc1,k3yInSc1,k3yQnSc1hP
# kdo to nevěděl
# komu jsem to nedal
# na koho naléhal

clauses = terminated_indicative_clauses + subordinate_clauses

rules = clauses + relatives

# věty:
# samostatné   oznamovací                             CLAUSE (MATRIX_CLAUSE)
#              tázací (s tázacím … na začátku)        INTERROGATIVE_CLAUSE
#              přací (s nějakou částicí na začátku)   ?
# nesamostatné přívlastkové                         ATTR_CLAUSE
#              podmětné/předmětné                   ?
#              adverbiální (časové, místní, způsobové atd.)

# Učím Petra angličtinu (akuzativ životný a neživotný)
# Petr půjčil Pavlovi knihu
# Kočka se dychtivě napila
# Chce/musí to odevzdat (zítra)
# T0D0: dá se udělat m="[^F]" a zároveň kopírovat modus do CLAUSE?
# '(CLAUSE e g n a) → (MODAL m="[^F]") (k=5 e g n a m=F)',
# lepší je parametrizovat až (CLAUSE m=[^F])

kombinace = '''
Chtěli obědvat
Chtěli jsme obědvat
Naši chtěli obědvat
V sobotu chtěli obědvat
Chtěli obědvat až po nás
Naši nechtěli ani obědvat

musím jít domů
domů prý jít musím
'''

_rules = [
    # Bude to odevzdávat až za měsíc.
    '(CLAUSE e g n a m=B) → (k=5 lemma="být" e g n a m=B) (VP m=F a=I)',

    # Bude to muset odevzdat/odevzdávat až za měsíc.
    '(CLAUSE e g n a m=B) → (k=5 lemma="být" e g n a m=B) (VP e g n a m=F)',

    # Umožňuje nám stanovit příznivější ceny pro domácí klienty.
    '(CLAUSE a e g m n) → (k=5 a e g m n) (VP m=F a=P)',

    # Chtěli [jsme tam] jít, ale nakonec [jsme si to] rozmysleli.

    # Chtěl jsem [lyžovat nebo bruslit].
    # (INF) → (INF) (k=8) (INF)
    # (INF) → (k=5 m=F)
    # (INF) → (k=5) (CLITICS) (NP)
    # (CLAUSE) → (MODAL) (CLITICS) (INF)
    # (NP) → (k=3 x=D) (NP)
    # (NP) → (k=1)
]

# primary_positions → secondary_positions
# facultative? (optionals?) (OPTIONALS)? → (NP c=4)
# rules = insert_gradually_into_rules(verbals, '(NP c=4)')

# spony (a jiný odpad):
# – ten generátor bude muset upravovat obě strany pravidla
# – přidá třeba OBJ_ACC a OBJ_ACC2/EXTRA (víc jich nečekám):
#   (VP OBJ_ACC OBJ_ACC2) → (k=5 m="[^F]") (OBJ_ACC)        (OBJ_ACC2)
#                               učím     Petra (+ANIM)   angličtinu (−ANIM)
# – životnost bohužel u ženskýho rodu nevím, to je pěkně blbý
#   musela by se brát z WordNetu, majka to prostě mít nebude

# '(COPULA e g n a m) → (lemma="být" e g n a m)',

# (všichni) jsme hladoví
# '(VP e g n a m) → (lemma="být" e g n a m) (ADJP)',

# TODO: má smysl zpracovávat spony odděleně od ostatních sloves, protože
# ostatní nebudou mít „Petr je učitel“ (nominativ)

# On je pytel blech.
# '(VP e g n a m) → (lemma="být" e g n a m) (NP)',  # je [něco]
